import { UndoableEdit } from "./UndoableEdit";
/**
 * The UndoManager keeps track of all editables.
 */
export declare class UndoManager {
    private edits;
    private position;
    private unmodifiedPosition;
    private limit;
    private listener;
    /**
     * Create a new UndoManager
     *
     * @param limit The maximum amount of editables to remember
     */
    constructor(limit?: number);
    /**
     * The listener will be called when changes have been done (adding an edit, or undoing/redoing it)
     *
     * @param listener The new callback or null to remove the existing one.
     */
    setListener(listener: null | (() => void)): void;
    /**
     * Test if there is anything to be saved
     */
    isModified(): boolean;
    /**
     * Mark the point when the data has been saved.
     */
    setUnmodified(): void;
    /**
     * Get the maximum amount of editables to remember
     */
    getLimit(): number;
    /**
     * Set the maximum amount of editables to remember.
     * The new limit will be applied instantly.
     *
     * @param value The maximum amount of editables to remember
     */
    setLimit(value: number): void;
    /**
     * Clear all edits
     */
    clear(): void;
    private applyLimit;
    /**
     * Test to see the new position after an undo would happen.
     *
     * @param position The start position
     * @return False if no significant edit can be undone.
     * Otherwise the new position.
     */
    private testUndo;
    /**
     * @returns true if there is anything to be undone (only significant edits count)
     */
    canUndo(): boolean;
    /**
     * Undo the last significant edit.
     * This will undo all insignificant edits up to the edit to be undone.
     *
     * @throws Error if no edit can be undone.
     */
    undo(): void;
    /**
     * Test to see the new position after an redo would happen.
     *
     * @param position The start position
     * @return False if no significant edit can be redone.
     * Otherwise the new position.
     */
    private testRedo;
    /**
     * @returns true if there is anything to be redone (only significant edits count)
     */
    canRedo(): boolean;
    /**
     * Redo the next significant edit.
     * This will redo all insignificant edits up to the edit to be redone.
     *
     * @throws Error if no edit can be redone.
     */
    redo(): void;
    /**
     * Add a new edit. Will try to merge or replace existing edits.
     *
     * @param edit The new edit to add.
     */
    add(edit: UndoableEdit): void;
}
